SystemOrganization addCategory: #'Zinc-FileSystem-Legacy'!

Object subclass: #ZnFileSystemUtils
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Zinc-FileSystem-Legacy'!

!ZnFileSystemUtils commentStamp: '<historical>' prior: 0!
I am ZnFileSystemUtils.
I help is abstracting over API differences pre and post the introduction of FileSystem in Pharo 2.0

Part of Zinc HTTP Components.!

!ZnFileSystemUtils class methodsFor: 'paths' stamp: 'SvenVanCaekenberghe 7/2/2012 16:10'!
baseNameFor: fileName
	^ (FileDirectory root directoryEntryFor: fileName) name! !

!ZnFileSystemUtils class methodsFor: 'defaults' stamp: 'SvenVanCaekenberghe 7/3/2012 00:37'!
defaultDirectory
	^ FileDirectory default! !

!ZnFileSystemUtils class methodsFor: 'defaults' stamp: 'SvenVanCaekenberghe 7/2/2012 22:36'!
defaultDirectoryPath
	^ FileDirectory default fullName! !

!ZnFileSystemUtils class methodsFor: 'paths' stamp: 'SvenVanCaekenberghe 7/2/2012 16:44'!
delete: path
	^ FileDirectory default deleteFileNamed: path! !

!ZnFileSystemUtils class methodsFor: 'paths' stamp: 'SvenVanCaekenberghe 7/2/2012 16:57'!
deleteIfExists: path
	^ (self exists: path)
		ifTrue: [ self delete: path. true ]
		ifFalse: [ false ]! !

!ZnFileSystemUtils class methodsFor: 'paths' stamp: 'SvenVanCaekenberghe 7/2/2012 16:31'!
downloadPathTo: directoryOfFileName for: baseName
	| downloadPath |
	downloadPath := FileDirectory on: directoryOfFileName.
	^ downloadPath exists
		ifTrue: [ (downloadPath / baseName) pathName ]
		ifFalse: [ directoryOfFileName ]
	! !

!ZnFileSystemUtils class methodsFor: 'paths' stamp: 'SvenVanCaekenberghe 7/2/2012 16:43'!
exists: path
	^ FileDirectory default fileExists: path! !

!ZnFileSystemUtils class methodsFor: 'paths' stamp: 'SvenVanCaekenberghe 7/2/2012 16:01'!
extensionFor: fileName
	^ FileDirectory extensionFor: fileName! !

!ZnFileSystemUtils class methodsFor: 'streams' stamp: 'SvenVanCaekenberghe 7/2/2012 16:23'!
fileNamed: fileName do: block
	^ FileStream 
		fileNamed: fileName 
		do: block! !

!ZnFileSystemUtils class methodsFor: 'queries' stamp: 'SvenVanCaekenberghe 7/2/2012 16:08'!
fileSizeFor: fileName
	^ (FileDirectory root directoryEntryFor: fileName) fileSize! !

!ZnFileSystemUtils class methodsFor: 'streams' stamp: 'SvenVanCaekenberghe 7/2/2012 16:13'!
fileStreamFor: fileName
	^ FileStream fileNamed: fileName! !

!ZnFileSystemUtils class methodsFor: 'paths' stamp: 'SvenVanCaekenberghe 7/3/2012 00:34'!
fullNameFor: fileName
	^ (FileDirectory default / fileName) fullName! !

!ZnFileSystemUtils class methodsFor: 'queries' stamp: 'SvenVanCaekenberghe 7/3/2012 00:40'!
modificationTimeFor: path
	^ (FileDirectory default directoryEntryFor: path) modificationDateAndTime! !

!ZnFileSystemUtils class methodsFor: 'streams' stamp: 'SvenVanCaekenberghe 7/3/2012 00:32'!
newFileNamed: fileName do: block
	^ FileStream 
		newFileNamed: fileName 
		do: block! !

!ZnFileSystemUtils class methodsFor: 'streams' stamp: 'SvenVanCaekenberghe 7/2/2012 22:33'!
oldFileNamed: fileName do: block
	^ FileStream 
		oldFileNamed: fileName 
		do: block! !

!ZnFileSystemUtils class methodsFor: 'streams' stamp: 'SvenVanCaekenberghe 7/2/2012 16:13'!
oldFileStreamFor: fileName
	^ FileStream oldFileNamed: fileName! !

Object subclass: #ZnStaticFileServerDelegate
	instanceVariableNames: 'prefix directory mimeTypeExpirations'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Zinc-FileSystem-Legacy'!

!ZnStaticFileServerDelegate commentStamp: 'SvenVanCaekenberghe 4/26/2011 12:48' prior: 0!
I am ZnStaticFileServerDelegate.
I am a simple proof of concept implementation of a web server serving static files.
I handle urls with an optional prefix as requests for files in a directory.
I serve index.html or index.htm when a directory is requested and these files exist.
I do a redirect when a path that is not does not end with a / refers to directory.
I function as a delegate for ZnServer.

ZnServer startDefaultOn: 1701.
ZnServer default delegate: ((ZnStaticFileServerDelegate new) 
									prefixFromString: 'apple/macosx'; 
									directory: (FileDirectory on: '/Library/WebServer/Documents'); 
									yourself).

Part of Zinc HTTP Components.!

!ZnStaticFileServerDelegate class methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 5/14/2012 10:26'!
defaultMimeTypeExpirations
	"Return a default dictionary mapping ZnMimeTypes to their expiration time in seconds.
	Missing entries will have no expiration time."

	| dict |
	dict := Dictionary new.
	dict at: ZnMimeType textCss put: 30 days asSeconds.
	dict at: ZnMimeType textJavascript put: 30 days asSeconds.
	dict at: ZnMimeType textPlain put: 30 days asSeconds.
	dict at: ZnMimeType textHtml put: 30 days asSeconds.
	dict at: ZnMimeType imageGif put: 30 days asSeconds.
	dict at: ZnMimeType imageJpeg put: 30 days asSeconds.
	dict at: ZnMimeType imagePng put: 30 days asSeconds.
	dict at: ZnMimeType applicationJavascript put: 30 days asSeconds.
	^ dict! !

!ZnStaticFileServerDelegate methodsFor: 'private' stamp: 'SvenVanCaekenberghe 5/15/2012 11:02'!
actualFilenameFor: uri
	| subElements subDir entry |
	(uri isEmpty and: [ self prefix isEmpty ]) ifTrue: [ ^ self indexFileIn: self directory ].
	(self prefix isEmpty or: [ uri isEmpty not and: [ uri pathSegments beginsWith: self prefix ] ]) ifFalse: [ ^ nil ].
	subElements := (uri pathSegments allButFirst: self prefix size) reject: [ :each | each = #/ ].
	subDir := subElements allButLast inject: self directory into: [ :parent :sub |
		(parent directoryExists: sub)
			ifTrue: [ parent directoryNamed: sub ]
			ifFalse: [ ^ nil ] ]. 
	subElements isEmpty
		ifTrue: [ entry := subDir containingDirectory directoryEntryFor: subDir localName ] 
		ifFalse: [ (entry := subDir directoryEntryFor: subElements last) ifNil: [ ^ nil ] ].
	^ entry isDirectory 
		ifTrue: [ self indexFileIn: entry asFileDirectory ]
		ifFalse: [ entry fullName ]! !

!ZnStaticFileServerDelegate methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 4/26/2011 11:06'!
directory
	"The FileDirectory whose files I am serving"
	
	^ directory ! !

!ZnStaticFileServerDelegate methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 4/26/2011 11:07'!
directory: fileDirectory
	"Set the FileDirectory whose files I should be serving"

	directory := fileDirectory! !

!ZnStaticFileServerDelegate methodsFor: 'private' stamp: 'SvenVanCaekenberghe 4/26/2011 12:35'!
directoryRedirectFor: uri
	| directoryUri |
	directoryUri := uri copy addPathSegment: #/.  
	^ ZnResponse redirect: directoryUri ! !

!ZnStaticFileServerDelegate methodsFor: 'private' stamp: 'SvenVanCaekenberghe 5/14/2012 10:30'!
expirationDateFor: aMimeType
	| expiration |
	expiration := self mimeTypeExpirations at: aMimeType ifAbsent: [ ^ nil ].
	^ ZnUtils httpDate: (DateAndTime now asUTC + expiration seconds)! !

!ZnStaticFileServerDelegate methodsFor: 'public' stamp: 'PaulDeBruicker 5/12/2012 09:56'!
handleRequest: request
	"Server delegate entry point"

	| actualFilename |
	(#(#GET #HEAD) includes: request method)
		ifFalse: [ ^ ZnResponse methodNotAllowed: request ].
	actualFilename := self actualFilenameFor: request uri.
	^ actualFilename notNil
		ifTrue: [ 
			(self redirectNeededFor: request uri actualFilename: actualFilename)
				ifTrue: [ self directoryRedirectFor: request uri ]
				ifFalse: [ self responseForFile: actualFilename fromRequest: request ] ]
		ifFalse: [ ZnResponse notFound: request uri ]! !

!ZnStaticFileServerDelegate methodsFor: 'private' stamp: 'SvenVanCaekenberghe 8/18/2011 14:25'!
indexFileIn: fileDirectory
	#( 'index.html' 'index.htm' ) do: [ :each |
			(fileDirectory fileExists: each)
				ifTrue: [ 
					^ (fileDirectory directoryEntryFor: each) fullName ] ].
	^ nil
! !

!ZnStaticFileServerDelegate methodsFor: 'private' stamp: 'SvenVanCaekenberghe 5/14/2012 09:41'!
maxAgeFor: aMimeType
	| expiration |
	expiration := self mimeTypeExpirations at: aMimeType ifAbsent: [ ^ nil ].
	^ 'max-age=', expiration asString! !

!ZnStaticFileServerDelegate methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 5/14/2012 09:39'!
mimeTypeExpirations
	"Return a dictionary mapping ZnMimeTypes to their expiration time in seconds.
	Missing entries will have no expiration time."

	^ mimeTypeExpirations ifNil: [ mimeTypeExpirations := Dictionary new ]! !

!ZnStaticFileServerDelegate methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 5/14/2012 09:39'!
mimeTypeExpirations: aDictionary
	"Set the mapping of ZnMimeTypes to their expiration time in seconds to aDictionary.
	Missing entries will have no expiration time."

	mimeTypeExpirations := aDictionary! !

!ZnStaticFileServerDelegate methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 4/26/2011 12:46'!
prefix
	"The path prefix collection under which I am serving files"
	
	^ prefix ifNil: [ prefix := #() ]! !

!ZnStaticFileServerDelegate methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 4/26/2011 11:08'!
prefix: orderedCollection
	"Set the path prefix under which I should be serving files to orderedCollection containing path elements"
	
	prefix := orderedCollection! !

!ZnStaticFileServerDelegate methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 4/26/2011 11:09'!
prefixFromString: string
	"Set the path prefix under which I should be serving files to string,
	interpreting each /-separated token as a path element"

	self prefix: (string findTokens: '/')! !

!ZnStaticFileServerDelegate methodsFor: 'private' stamp: 'SvenVanCaekenberghe 4/26/2011 13:11'!
redirectNeededFor: uri actualFilename: actualFilename
	uri isDirectory ifTrue: [ ^ false ].  
	^ (actualFilename endsWith: uri lastPathSegment) not! !

!ZnStaticFileServerDelegate methodsFor: 'private' stamp: 'SvenVanCaekenberghe 5/14/2012 09:53'!
responseForFile: filename fromRequest: aRequest
	| entry size time mimeType fileStream entity response modified |
	entry := FileDirectory root directoryEntryFor: filename.
	size := entry fileSize.
	time := entry modificationDateAndTime.
	modified := true.
	aRequest headers
		at: 'If-Modified-Since'
		ifPresent: [ modified := time > (ZnUtils parseHttpDate: (aRequest headers at: 'If-Modified-Since')) ].
	modified
		ifTrue: [ 
			mimeType := ZnMimeType forFilenameExtension: (FileDirectory extensionFor: filename).
			fileStream := FileDirectory root oldFileNamed: filename.
			mimeType isBinary
				ifTrue: [ fileStream binary ].
			(entity := ZnStreamingEntity type: mimeType length: size) stream: fileStream.
			(response := ZnResponse ok: entity) 
				headers at: 'Modification-Date' put: (ZnUtils httpDate: time).
			(self maxAgeFor: mimeType) 
				ifNotNil: [ :maxAge | response headers at: 'Cache-Control' put: maxAge ].
			(self expirationDateFor: mimeType)
				ifNotNil: [ :expirationDate | response headers at: 'Expires' put: expirationDate ] ]
		ifFalse: [ response := ZnResponse notModified ].
	^ response! !
